/*
 *  MOST:  help.c
 *
 *  Modified for C SWING and GOPHER compatibility by Foteos Macrides
 *                                                MACRIDES@SCI.WFEB.EDU
 */

#include <stdio.h>
#ifdef VMS
#include <stat.h>
#else
#include <sys/types.h>
#include <sys/stat.h>
#endif
#include "window.h"
#include "file.h"
#include "display.h"
#include "line.h"
#include "externs.h"
#include "sysdep.h"

/* 
** Quick Help section provided by Mats Akerberg (mats@efd.lth.se)
** Modified by Foteos Macrides
*/

static char *help[] = {
"  Starred commands may be repeated 'n' times by preceding the command with",
"  a number. EDT-style commands may be substituted. GOLD (Gd) is PF1 or ^X",
"       (numbers after Gd may be entered on the keypad or keyboard).",
"",
"N,NEXT  *Next page            DOWN,D,J *Down one line       T  Top of file",
"P,PREV  *Previous page          UP,U,K *Up one line         B  Bottom of file",
">,RIGHT *59 columns Right         [#]G  Go to line #        W  toggle Width",
"<,LEFT  *59 columns Left          [#]%  go to percent #     E  invoke Editor",
"Q,X,^Z   Quit from MOST        PF2,H,?  quick Help          $  spawn to VMS",
" ^R,^W   Redraw the screen   GdPF1,GdH  full Help    GdRETURN  system command",
"",
" / *search FORWARD for a string       PF3,R *Repeat current search",
" \\ *search REVERSE for a string         KP4  set FORWARD",
":C  toggle Case-sensitive search        KP5  set REVERSE",
"^G  abandon prompt (cancel)        SELECT,M  set Mark",
"^U  erase prompt string            INSERT,.  set mark and go to previous mark",
":D  Delete file (if +d set)                  (if present, for toggling pages)",
"",
"Gd2  split this window in half            O,GdO *move to Other window",
" :N  skip to Next file in the file ring     Gd0  delete this window",
"       (UP/DOWN arrow keys cycle ring)      Gd1  delete all other windows",
"GdG  Get a new file (if not SecureMode)       L  toggle window Lock",
"",
"[#]:O  toggle Options:  b Binary<->ASCII   i skip lines Indented > # columns,",
"          s Squeeze multiple blank lines   t show Tabs as ^I   v show ^chars,",
"          w Wrapping for current window     (v must be set for t to be set)",
"",
NULL };

/*
** display Quick Help
*/
void do_help_command()
{
    int i,ct,v_flag=0,w_flag=0;
    char *beg, *end, ch;
    char line[80];
    char attr[80];

    i = 1;
    ct = 0;
    set_scroll_region(1,SCREEN_HEIGHT);
    if (MOST_V_OPT)
      {
          v_flag = MOST_V_OPT;
          MOST_V_OPT = 0;
      }
    if (MOST_W_OPT)
      {
          w_flag = MOST_W_OPT;
          MOST_W_OPT = 0;
      }
    cls();
    while( i <= SCREEN_HEIGHT)
      {
          if( help[ct]!=NULL)
            {
                beg = help[ct];
                end = (help[ct] + strlen(help[ct++]));
                analyse_line((unsigned char *) beg,
                             (unsigned char *) end,
                             line, attr);
                output(line,end - beg,attr,'>');
                fputc('\n',stdout);
                i++;
		if (i == SCREEN_ROWS && help[ct+1] == NULL) i++;
                else if (i == SCREEN_HEIGHT && help[ct] != NULL)
                  {
		     set_attribute(7);
                     fputs("Press RETURN for more...",stdout);
		     set_attribute(0);
                     fflush(stdout);
		     while ((ch = getkey()) != '\015' && ch != '\012') ;
                     cls();
                     i = 1;
                  }
            }
          else
            {
		set_attribute(7);
                fputs("Press RETURN to continue.",stdout);
		set_attribute(0);
                fflush(stdout);
		while ((ch = getkey()) != '\015' && ch != '\012') ;
                if (w_flag)
                  MOST_W_OPT = w_flag;
                if (v_flag)
                  MOST_V_OPT = v_flag;
                redraw_display();
		return;
            }
          fflush(stdout);
      }
}

/*
** display Full Help (most.doc_fm)
*/
int do_help_doc_command()
{
    char *helpfile ,*buf_name;

    /*
    ** do we know where the doc file is?
    */
    if (!strlen(MOST_DOCFILE))
         most_message
("Set the environment variable 'MOST_HELP' to point to the most.doc_fm file."
 ,1);

    /*
    ** were we called with the -h switch?
    */
    else if (MOST_H_OPT)
         most_message("You are reading the Full Help file!", 1);

#ifdef VMS
    /*
    ** if most is defined as a foreign command,
    **   spawn "most -h" to make the window full screen
    */
    else if (NULL != getenv("most"))
      {
         if (reset_most_tty())
	   {
              reset_display();
              fflush(stdout);
              cls();
         
              if (!do_most_system_command("most -h", 0))
                   most_message("Full Help fetch failed.", 1);
         
              if (init_most_tty())
                   redraw_display();
              else
                {
                   fputs("\nError re-initializing the terminal!!!\n", stderr);
                   return (-1);
                }
           }
      }
#endif

    /*
    ** this uses a split window presentation like that in Davis' most
    */
    else
      {
         buf_name =
            "MOST.DOC_FM  Use GOLD 0 to delete this window, H for Quick Help ";

         if (file_visible(buf_name))
	   {
              most_message("You are reading the Full Help file!", 1);
              return(0);
           }
         if (!split_window())
           {
               most_message("Too many windows.  Cannot insert Full Help.",1);
               return(0);
           }
         update_status(1);  /* create status line of prev. window */
         other_window(1);
         find_file(MOST_DOCFILE);
         strcpy(BUF->file,buf_name);
         window_buffer();
         redraw_window();
         update_status(1);
      }
      
      return(0);
}
